
use master
go
create database simple
go
alter database simple add filegroup simple_mod contains memory_optimized_data
go
-- adapt filename as needed
alter database simple add file (name='simple_mod', filename='c:\data\simple_mod') to filegroup simple_mod
go

use simple
go
create table dbo.t1
(c1 int not null primary key nonclustered hash with (bucket_count=10000),
 c2 varchar(10))
with (memory_optimized=on)
go

insert dbo.t1 values(1, 'Ann'),(2, 'Bob'),(3, 'Cathy'),(4, 'Dot')
go

create table dbo.t2
(c1 int not null primary key nonclustered hash with (bucket_count=10000),
 c2 varchar(10))
with (memory_optimized=on)
go

insert dbo.t2 values(1, 'Ann'),(2, 'Bob'),(3, 'Cathy'),(4, 'Dot')
go

-- transaction isolation level

-- The database simple SNAPSHOT and RCSI turned off at the database level
-- I'm running at default isolation level of read committed
-- XACT_ABORT session setting is turned off
select * from sys.databases
dbcc useroptions
select @@trancount -- (for your convenience while running the demos)

--1. SNAPSHOT
begin tran
select * from dbo.t1 with (snapshot)
-- update a row in another session
commit -- OK
go

begin tran
select * from dbo.t1 with (snapshot)
-- update a row in another session
select * from dbo.t1 with (snapshot) -- sees old value
commit -- OK

--2. REPEATABLE READ
begin tran
select * from dbo.t1 with (repeatableread)
-- update a row in another session
commit -- Msg 41305, Level 16, State 0, Line 39 The current transaction failed to commit due to a repeatable read validation failure.

begin tran
select * from dbo.t1 with (repeatableread)
-- insert a row in another session
commit -- OK

begin tran
select * from dbo.t1 with (repeatableread)
-- insert a row in another session
select * from dbo.t1 with (repeatableread) -- does not see new row
commit -- OK

begin tran
-- update row in another session before reading
select * from dbo.t1 with (repeatableread) -- sees new value
commit -- OK

begin tran
select * from dbo.t2 with (repeatableread) -- transaction starts with first SQL statement
-- update row in t1 in another session before reading
select * from dbo.t1 with (repeatableread)
commit -- Msg 41305, Level 16, State 0, Line 72 The current transaction failed to commit due to a repeatable read validation failure.

-- 3. SERIALIZABLE
begin tran
select * from dbo.t1 with (serializable)
-- insert a row in another session
commit -- OK Msg 41325, Level 16, State 0, Line 55 The current transaction failed to commit due to a serializable validation failure.

-- More stuff...

begin tran
select * from dbo.t1 with (repeatableread)
-- insert a row in another session
select * from dbo.t1 with (serializable) -- does not see new row
commit -- OK  Msg 41325, Level 16, State 0, Line 61 The current transaction failed to commit due to a serializable validation failure.

begin tran
insert dbo.t1 values(9, 'Isaac')
-- insert row with same primary key in another session and commit other session (this works)
commit -- Msg 41325, Level 16, State 0, Line 66 The current transaction failed to commit due to a serializable validation failure.

begin tran
select * from dbo.t1 with (readcommitted) -- the table option 'readcommitted' is not supported with memory optimized tables.
commit -- OK
go

begin tran
select * from dbo.t1 -- Msg 41368, Level 16, State 0, Line 37 Accessing memory optimized tables using the READ COMMITTED isolation level is supported only for autocommit transactions. It is not supported for explicit or implicit transactions. Provide a supported isolation level for the memory optimized table using a table hint, such as WITH (SNAPSHOT).
commit -- no corresponding transaction


